<?php
/*
Plugin Name: WP low Profiler
Plugin URI: http://robertmahfoud.konceptus.net/posts/wp-low-profiler/
Description: Enables a user to create low profile posts/pages that are not published on the homepages, category pages and/or feeds, but at the same accessible by direct permalink, search or, for posts, through the archives, and also visible to search engines.
Version: 1.0.6
Author: Robert Mahfoud
Author URI: http://robertmahfoud.konceptus.net
Text Domain: wp_low_profiler
*/

/*  Copyright 2009  Robert Mahfoud  (email : robert.mahfoud@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// Create Text Domain For Translations
add_action('init', 'wplp_textdomain');
function wplp_textdomain() {
	$plugin_dir = basename(dirname(__FILE__));
	load_plugin_textdomain('wp-low-profiler', ABSPATH."/$plugin_dir", $plugin_dir);
}

// Activating the plugin
function wplp_activate() {
	if(@is_file(ABSPATH.'/wp-admin/upgrade-functions.php')) {
		include_once(ABSPATH.'/wp-admin/upgrade-functions.php');
	} elseif(@is_file(ABSPATH.'/wp-admin/includes/upgrade.php')) {
		include_once(ABSPATH.'/wp-admin/includes/upgrade.php');
	} else {
		die('We have problem finding your \'/wp-admin/upgrade-functions.php\' and \'/wp-admin/includes/upgrade.php\'');
	}
	
	// Create Low Profiler Posts Table
	$create_sql = "CREATE TABLE  wp_lowprofiler_posts (".
							  "wplp_post_id bigint(20) NOT NULL,".
							  "wplp_flag varchar(255) NOT NULL,".
							  "wplp_value varchar(255) NULL,".
							  "PRIMARY KEY (wplp_id, wplp_flag)".
							")";
	maybe_create_table(wp_lowprofiler_posts, $create_sql);	
}
register_activation_hook( __FILE__, 'wplp_activate' );

function wplp_update_low_profile($id, $lp_flag, $lp_value) {
	$item = get_post_type($id);
	global $wpdb;
	if( ($item == 'post' && !$lp_value) || ($item == 'page' && (!$lp_value || $lp_value == 'none')) ) {
		wplp_unset_low_profile($item, $id, $lp_flag);
	} else {
		wplp_set_low_profile($item, $id, $lp_flag, $lp_value);
	}
}

function wplp_unset_low_profile($item, $id, $lp_flag) {
	global $wpdb;
	// Delete the flag from the database table
	$wpdb->query("DELETE FROM wp_lowprofiler_posts WHERE wplp_post_id = $id AND wplp_flag = '$lp_flag'");
}

function wplp_set_low_profile($item, $id, $lp_flag, $lp_value) {
	global $wpdb;	
	// Ensure No Duplicates!
	$check = strval($wpdb->get_var("SELECT wplp_flag FROM wp_lowprofiler_posts WHERE wplp_post_id = $id AND wplp_flag='$lp_flag'"));
	if(!$check) {
		$wpdb->query("INSERT INTO wp_lowprofiler_posts(wplp_post_id, wplp_flag, wplp_value) VALUES($id, '$lp_flag', '$lp_value')");
	} elseif( $item == 'page' ) {
		$wpdb->query("UPDATE wp_lowprofiler_posts set wplp_value = '$lp_value' WHERE wplp_post_id = $id AND wplp_flag = 'page'");
	}
}

// hook function to filter out low profile pages
function wplp_exclude_low_profile_pages($posts) {
	return wplp_exclude_low_profile_items('page', $posts);
}

function wplp_exclude_low_profile_items($item, $posts) {
	if( !is_admin() ) {
		if( $item != 'page' )
			return $posts;
		else {
			global $wpdb;
			// now loop over the pages, and exclude the ones with low profile in this context
			$result = array();
			foreach($posts as $post) {
				$check = strval($wpdb->get_var("SELECT wplp_value FROM wp_lowprofiler_posts WHERE wplp_post_id = $post->ID AND wplp_flag = 'page'"));
				if( ($check == 'home' && is_home()) || $check == 'all') {
					// exclude page
				} else
					$result[] = $post;
			}
			return $result;
		}
	}
}
// register the filter "the_posts" to hide low profile posts
add_filter('get_pages', 'wplp_exclude_low_profile_pages');

add_action('admin_menu', 'wplp_add_post_edit_meta_box');
function wplp_add_post_edit_meta_box() {
	add_meta_box('lowprofilerdivpost', __('Low Profile Attributes', 'wp-low-profiler'), 'wplp_metabox_post_edit', 'post', 'side');
	add_meta_box('lowprofilerdivpage', __('Low Profile Attributes', 'wp-low-profiler'), 'wplp_metabox_page_edit', 'page', 'side');
}

function wplp_metabox_post_edit() {
	global $wpdb;
	
	$id = isset($_GET['post']) ? intval($_GET['post']) : 0;

	$wplp_home = 0;
	$wplp_category = 0;
	$wplp_feed = 0;
	
	if($id > 0) {
		$flags = $wpdb->get_results("SELECT wplp_flag from wp_lowprofiler_posts where wplp_post_id = $id", ARRAY_N);
		if( $flags ) {
			foreach($flags as $flag_array) {
				$flag = $flag_array[0];
				${"wplp_$flag"} = 1;
			} 
		}
	}
?>
	<label for="wplp_home" class="selectit"><input type="checkbox" id="wplp_home" name="wplp_home" value="1"<?php checked($wplp_home, 1); ?>/>&nbsp;<?php _e('Hide on home page(s).', 'wp-low-profiler'); ?></label>
	<br />
	<label for="wplp_category" class="selectit"><input type="checkbox" id="wplp_category" name="wplp_category" value="1"<?php checked($wplp_category, 1); ?>/>&nbsp;<?php _e('Hide on category page(s).', 'wp-low-profiler'); ?></label>
	<br />
	<label for="wplp_feed" class="selectit"><input type="checkbox" id="wplp_feed" name="wplp_feed" value="1"<?php checked($wplp_feed, 1); ?>/>&nbsp;<?php _e('Hide in feed(s).', 'wp-low-profiler'); ?></label>
<?php
}

function wplp_metabox_page_edit() {
	global $wpdb;
	
	$id = isset($_GET['post']) ? intval($_GET['post']) : 0;

	$wplp_page = 'none';
	
	if($id > 0) {
		$flags = $wpdb->get_results("SELECT wplp_value from wp_lowprofiler_posts where wplp_post_id = $id AND wplp_flag = 'page'", ARRAY_N);
		if( $flags )
			$wplp_page = $flags[0][0];
	}
?>
	<label class="selectit"><input type="radio" id="wplp_page_none" name="wplp_page" value="none"<?php checked($wplp_page, 'none'); ?>/>&nbsp;<?php _e('Show normally everywhere.', 'wp-low-profiler'); ?></label>
	<br />
	<br />
	<label class="selectit"><input type="radio" id="wplp_page_all" name="wplp_page" value="all"<?php checked($wplp_page, 'all'); ?>/>&nbsp;<?php _e('Hide everywhere pages are listed (except the sitemap.xml if you generate one automatically).', 'wp-low-profiler'); ?></label>
	<br />
	<br />
	<label class="selectit"><input type="radio" id="wplp_page_home" name="wplp_page" value="home"<?php checked($wplp_page, 'home'); ?>/>&nbsp;<?php _e('Hide when listing pages on home page(s) only.', 'wp-low-profiler'); ?></label>
<?php
}

function wplp_save_post($id) {
	$item = get_post_type($id);
	if( $item == 'post' ) {
		$wplp_home = isset($_POST['wplp_home']) ? $_POST['wplp_home'] : 0;
		$wplp_category = isset($_POST['wplp_category']) ? $_POST['wplp_category'] : 0;
		$wplp_feed = isset($_POST['wplp_feed']) ? $_POST['wplp_feed'] : 0;
		
		wplp_update_low_profile($id, 'home', $wplp_home);
		wplp_update_low_profile($id, 'category', $wplp_category);
		wplp_update_low_profile($id, 'feed', $wplp_feed);
	} elseif( $item == 'page' ) {
		$wplp_page = isset($_POST['wplp_page']) ? $_POST['wplp_page'] : 'none';
		wplp_update_low_profile($id, 'page', $wplp_page);
	}	
}
add_action('save_post', 'wplp_save_post');

function wplp_delete_post($post_id) {
	global $wpdb;
	// Delete all post flags from the database table
	$wpdb->query("DELETE FROM wp_lowprofiler_posts WHERE wplp_id = $post_id");
}
add_action('delete_post', 'wplp_delete_post');

function wplp_query_posts_where($where) {
	// filter posts on one of the three kinds of contexts: home, category, feed
	if( !is_admin() && (is_feed() || is_home() || is_category()) ) {
		$where .= ' AND wp_lowprofiler_posts.wplp_post_id IS NULL ';
	}
	return $where;
}

function wplp_query_posts_join($join) {
	// filter posts on one of the three kinds of contexts: home, category, feed
	if( !is_admin() && (is_feed() || is_home() || is_category()) ) {

		if( !$join )
			$join = '';
	
		$join .= ' LEFT OUTER JOIN wp_lowprofiler_posts ON wp_posts.ID = wp_lowprofiler_posts.wplp_post_id ';
		// posts
		$join .= ' AND (wp_posts.post_type = \'post\' ';
		if(is_home())
			$join .= ' AND wp_lowprofiler_posts.wplp_flag NOT IN ( \'category\', \'feed\' ) ';
		elseif( is_category())
			$join .= ' AND wp_lowprofiler_posts.wplp_flag NOT IN ( \'home\', \'feed\' ) ';
		elseif( is_feed())
			$join .= ' AND wp_lowprofiler_posts.wplp_flag NOT IN ( \'home\', \'category\' ) ';
		$join .= ')';
	}		
	return $join;
}

add_filter('posts_where_paged', 'wplp_query_posts_where');
add_filter('posts_join_paged', 'wplp_query_posts_join');

?>